# -*- coding: utf-8 -*-

import re

def make_lengths_seq(words):
    for t, w in words:
        if t == 'w':
            wlen = len(w)
            yield 'w', wlen
        else:
            yield t, w

def split_to_words(text):
    for word in re.findall(u'(?iu)[а-я]+', text.lower()):
        syms = []
        if word.endswith(','):
            syms.append(',')

        word = word.strip(',')
        if word:
            yield 'w', word
            if syms:
                for s in syms:
                    yield 's', s

            yield 's', ' '


def run():
    import sys
    from optparse import OptionParser

    from typetrainer.generator import make_char_chain, generate_word


    parser = OptionParser(usage="usage: %prog [options] file_with_words")
    parser.add_option("-t", "--tutor", dest="tutor", default='en.basic',
        help="Tutor maker to use (en.basic, ru.basic)", metavar="tutor")

    options, args = parser.parse_args()

    def get_chains(tutor, filename):
        package_name = 'typetrainer.tutors.' + tutor
        try:
            __import__(package_name)
        except ImportError:
            parser.error("Can't fint [%s] tutor" % tutor)

        pkg = sys.modules[package_name]

        try:
            words = list(pkg.split_to_words(open(filename).read().decode('utf-8')))
        except IOError:
            parser.error("Can't read %s" % filename)

        first, other = make_char_chain(words, 3)
        lengths = list(pkg.make_lengths_seq(words))

        return first, other, lengths

    if not args:
        parser.error('You should specify file with words to process')

    first, other, lengths = get_chains(options.tutor, args[0])


    import gtk
    import random
    import itertools

    from typetrainer.ui import idle
    from typetrainer.ui.main import Main

    def filler():
        pos = random.randint(0, len(lengths) - 1)
        left = itertools.islice(lengths, pos, None)
        right = itertools.islice(lengths, 0, pos)

        for t, l in itertools.cycle(itertools.chain(left, right)):
            if t == 'w':
                yield generate_word(first, other, l, 3)
            else:
                yield l

    app = Main(filler)
    app.window.show()
    idle(app.fill)

    try:
        gtk.main()
    except KeyboardInterrupt:
        pass


import gobject

def idle_callback(callable, args):
    args, kwargs = args
    callable(*args, **kwargs)
    return False

def idle(callable, *args, **kwargs):
    options = {}
    if 'priority' in kwargs:
        options['priority'] = kwargs['priority']
        del kwargs['priority']
    return gobject.idle_add(idle_callback, callable, (args, kwargs), **options)


import os.path
import time

import gtk
import pango

def attach_glade(obj, filename, *names):
    builder = gtk.Builder()
    builder.add_from_file(filename)

    for name in names:
        setattr(obj, name, builder.get_object(name))

    builder.connect_signals(obj)

class Main(object):
    """glade-file: main.glade"""

    def __init__(self, filler):
        self.filler = filler

        attach_glade(self, os.path.join(os.path.dirname(__file__), 'main.glade'),
            'window', 'totype_entry', 'type_entry', 'cpm_lb', 'accuracy_lb')

    def fill(self):
        self.type_entry.set_text('')
        self.start_time = 0

        entry = self.totype_entry

        entry.modify_font(pango.FontDescription('sans 16'))
        self.type_entry.modify_font(pango.FontDescription('sans 16'))

        text = ''
        d = entry.get_layout_offsets()[0] * 2

        for w in self.filler():
            text += w

            entry.set_text(text)
            if entry.allocation.width - d < entry.get_layout().get_pixel_size()[0]:
                break

        while entry.allocation.width - d < entry.get_layout().get_pixel_size()[0]:
            text = text[:-1].strip()
            entry.set_text(text)

        self.type_entry.set_max_length(entry.get_text_length())

    def on_window_delete_event(self, *args):
        gtk.main_quit()

    def on_type_entry_changed(self, *args):
        if not self.start_time:
            self.start_time = time.time()

        ref_text = unicode(self.totype_entry.get_text())
        cur_text = unicode(self.type_entry.get_text())

        bad_idx = []
        for i, c in enumerate(cur_text):
            if c != ref_text[i]:
                bad_idx.append(i)

        attrs = pango.AttrList()
        for idx in bad_idx:
            # Pango layout needs byte index not char one O_o
            idx = len(ref_text[:idx].encode('utf8'))
            attrs.change(pango.AttrForeground(65535, 0, 0, idx, idx+1))

        self.totype_entry.get_layout().set_attributes(attrs)
        self.totype_entry.queue_draw()

    def on_type_entry_activate(self, *args):
        if self.start_time:
            delta = time.time() - self.start_time

            ref_text = unicode(self.totype_entry.get_text())
            cur_text = unicode(self.type_entry.get_text())

            self.cpm_lb.set_text("%d" % int((len(cur_text) + 1) * 60.0 / delta))

            errors = len(ref_text) - len(cur_text)
            for i, c in enumerate(cur_text):
                if c != ref_text[i]:
                    errors += 1

            self.accuracy_lb.set_text(
                '%d%%' % int((len(ref_text) - errors) * 100.0 / len(ref_text)))

        self.fill()


from random import choice

def make_char_chain(words, seq_len):
    first, other = {}, {}

    for t, w in words:
        if t != 'w': continue

        wlen = len(w)
        if wlen <= seq_len:
            first.setdefault(wlen, []).append(w)
        else:
            pc = w[:seq_len]
            first.setdefault('any', []).append(pc)
            pos = seq_len
            while True:
                next = w[pos:pos+seq_len]
                if not next:
                    break

                other.setdefault(pc, []).append(next)
                pc = next
                pos += seq_len

    return first, other

def chain_traversor(choices, other, length):
    copied = False

    while True:
        try:
            head = choice(choices)
            tail = get_tail(head, other, length)
            return [head] + tail
        except KeyError:
            if not copied:
                choices = choices[:]
                copied = True

            choices.remove(head)
            if not choices:
                raise KeyError()

def get_tail(head, other, length):
    hlen = len(head)
    if hlen == length:
        return []
    elif hlen > length:
        raise KeyError()

    choices = other[head]
    return chain_traversor(choices, other, length - hlen)

def generate_word(first, other, length, seq_length):
    if length <= seq_length:
        return choice(first[length])

    return ''.join(chain_traversor(first['any'], other, length))